#ifndef _sample_vci_h
#define _sample_vci_h

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include <sys/ipc.h>
#include <sys/shm.h>

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/

#define DEBUG	/* print more messages for debug */

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* System V IPC access permission */
#define ACCESS_PERMS	0600

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*
 * Structure used by the VCI plugin to send to the application
 * the Ids of the shared memory segment and of the semaphore.
 */
/* shmid and semid are System V IPC ids */
typedef struct {
	u_int width, height;	/* size of frame */
	int shmid[3];	/* shared memory segment id */
} SHARED_IDS;


/*
 * YUV_IMAGE_AND_INDEX can hold a YUV_IMAGE and an index.
 * In our example, each shared memory segment will start with
 * a YUV_IMAGE_AND_INDEX structure, followed immediately by the
 * Y, U and V planes.
 * The YUV_IMAGE will be a copy of the YUV_IMAGE of the frame
 * that has been captured (be careful: the plane pointers become
 * invalid when seen in the application address space).
 * The index will be used by our VCI_free to get the
 * shared memory segment Id when we are
 * asked to free the shared memory.
 */
typedef struct {
	YUV_IMAGE	yuv_image;
	int			index;
} YUV_IMAGE_AND_INDEX;


#endif /* _sample_vci_h */
